import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption,  } from 'app/core/request/request-util';
import { Search } from 'app/core/request/request.model';
import { IWatchListEntry } from 'app/shared/model/watch-list-entry.model';

type EntityResponseType = HttpResponse<IWatchListEntry>;
type EntityArrayResponseType = HttpResponse<IWatchListEntry[]>;

@Injectable({ providedIn: 'root' })
export class WatchListEntryService {
  public resourceUrl = this.applicationConfigService.getEndpointFor('api/watch-list-entries');
  public resourceUrlCurrentUser = this.applicationConfigService.getEndpointFor('api/currentuser-watch-list-entries');
  public resourceUrlCurrentUserWL = this.applicationConfigService.getEndpointFor('api/currentuser-watch-lists');
  public resourceSearchUrl = this.applicationConfigService.getEndpointFor('api/_search/watch-list-entries');

  constructor(protected http: HttpClient, private applicationConfigService: ApplicationConfigService) {}

  create(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.post<IWatchListEntry>(this.resourceUrl, watchListEntry, { observe: 'response' });
  }

  createForCurrentUser(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.post<IWatchListEntry>(this.resourceUrlCurrentUser, watchListEntry, { observe: 'response' });
  }

  deleteForCurrentUser(watchListId: number, exerciseId: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrlCurrentUserWL}/${watchListId}/entries/${exerciseId}`, { observe: 'response' });
  }
  update(watchListEntry: IWatchListEntry): Observable<EntityResponseType> {
    return this.http.put<IWatchListEntry>(this.resourceUrl, watchListEntry, { observe: 'response' });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IWatchListEntry>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IWatchListEntry[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  search(req: Search): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IWatchListEntry[]>(this.resourceSearchUrl, { params: options, observe: 'response' });
  }

  getWatchListContent(id: number): Observable<EntityArrayResponseType> {
    return this.http.get<IWatchListEntry[]>(`${this.resourceUrlCurrentUserWL}/${id}/entries`, { observe: 'response' });
  }
}
