import { Component } from '@angular/core';
import { Exercise, searchResultToExercise } from 'app/shared/model/exercise.model';
import { SearchInput } from 'app/shared/model/search/search-input.model';
import { SearchService } from 'app/search/service/search-service';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';
import { AccountService } from 'app/core/auth/account.service';

@Component({
  selector: 'jhi-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.scss'],
})
export class SearchComponent {
  results: Exercise[] = [];
  selectedResult?: Exercise;
  hitCount = 0;

  private searchInput: SearchInput = new SearchInput();

  constructor(protected searchService: SearchService, public accountService: AccountService) {}

  selectExercise(exercise: Exercise): void {
    this.selectedResult = exercise;
  }

  updateSearchInput(searchInput: SearchInput): void {
    searchInput.page = 0;
    this.searchInput = searchInput;
    this.results = [];
    this.hitCount = 0;
    this.search();
  }

  private search(): void {
    if (
      this.searchInput.fulltextQuery ||
      this.searchInput.metadata.programmingLanguage ||
      this.searchInput.metadata.keyword ||
      this.searchInput.metadata.author ||
      this.searchInput.metadata.license ||
      this.searchInput.metadata.format ||
      this.searchInput.metadata.naturalLanguage ||
      this.searchInput.metadata.learningResourceType
    ) {
      this.searchService.searchPageDetails(this.searchInput).subscribe(
        (data: SearchResultsDTO) => {
          const searchResults = this.parseSearchResultsDTO(data);
          this.results = this.results.concat(searchResults);
          ++this.searchInput.page;
        },
        () => alert('Search failed')
      );
    }
  }

  private parseSearchResultsDTO(searchResultsDTO: SearchResultsDTO): Exercise[] {
    this.hitCount = searchResultsDTO.hitCount;
    // fix string to date conversion: unfortunatelly dates are not converted correctly :-()
    searchResultsDTO.searchResult.forEach(hit => {
      hit.project.last_activity_at = new Date(hit.project.last_activity_at);
      hit.file.indexing_date = new Date(hit.file.indexing_date);
    });
    return searchResultsDTO.searchResult.map(searchResultToExercise);
  }

  onScroll(): void {
    this.search();
  }
}
