package at.ac.uibk.gitsearch;

import static com.tngtech.archunit.base.DescribedPredicate.alwaysTrue;
import static com.tngtech.archunit.core.domain.JavaClass.Predicates.belongToAnyOf;
import static com.tngtech.archunit.library.Architectures.layeredArchitecture;

import com.tngtech.archunit.base.DescribedPredicate;
import com.tngtech.archunit.core.domain.JavaClass;
import com.tngtech.archunit.core.importer.ImportOption.DoNotIncludeTests;
import com.tngtech.archunit.junit.AnalyzeClasses;
import com.tngtech.archunit.junit.ArchTest;
import com.tngtech.archunit.lang.ArchRule;

import at.ac.uibk.gitsearch.repository.gitlab.GitLabRepository;
import at.ac.uibk.gitsearch.security.jwt.TokenProvider;

@AnalyzeClasses(packagesOf = GitsearchApp.class, importOptions = DoNotIncludeTests.class)
class TechnicalStructureTest {

	static class ServiceLayer extends DescribedPredicate<JavaClass> {
		public ServiceLayer() {
			super("ServiceLayer");
		}

		@Override
		public boolean apply(JavaClass input) {
			return input.getPackageName().contains(".service") && !input.getPackageName().contains(".service.dto");
		}
	}
		
    // prettier-ignore
    @ArchTest
    static final ArchRule respectsTechnicalArchitectureLayers = layeredArchitecture()
        .layer("Config").definedBy("..config..")
        .layer("Web").definedBy("..web..")
        .layer("Service").definedBy(new ServiceLayer())
        .layer("DTO").definedBy("..service.dto..")
        .layer("Security").definedBy("..security..")
        .layer("Persistence").definedBy("..repository..")
        .layer("Domain").definedBy("..domain..")
        .layer("elasticSearch").definedBy("..es..")

        .whereLayer("Config").mayNotBeAccessedByAnyLayer()
        .whereLayer("Web").mayOnlyBeAccessedByLayers("Config")
        .whereLayer("Service").mayOnlyBeAccessedByLayers("Web", "Config")
        // temporary exceptions
           .ignoreDependency(GitLabRepository.class, TokenProvider.class)
           .ignoreDependency(GitLabRepository.class, TokenProvider.GitLabAccessInfo.class)
           
                      
        .whereLayer("Security").mayOnlyBeAccessedByLayers("Web", "Service", "Config")
        .whereLayer("Persistence").mayOnlyBeAccessedByLayers("Service", "Security", "Web", "Config")
        .whereLayer("Domain").mayOnlyBeAccessedByLayers("Persistence", "Service", "Security", "Web", "Config", "DTO")

        .ignoreDependency(belongToAnyOf(GitsearchApp.class), alwaysTrue());
}
